# cody.py (EXPLORER Hardware IR Test)
# Copyright: 2024 Bradan Lane STUDIO
#
# Open Source License: MIT

'''
This CircuitPython program is used to test the hardware of the EXPLORER board.
While the code does work, it may not serve as the best example of what could be considered "good code". 

Implementation Notes
--------------------
**Hardware:**
    * the Explorer Badge (eChallengeCoin 20204 and DC NextGen DC32)

**Software and Dependencies:**
    * CircuitPython firmware (UF2): https://circuitpython.org
    * Adafruit CircuitPython Libraries: https://circuitpython.org/libraries

**Open Source Repository:**
    * GitLab: https://gitlab.com/bradanlane

'''

import board            # provides access to all of the board IO pins and hardware
import time             # convenient for timing tasks and for 'sleep' delays
import random
import digitalio
import pulseio          # used for the IR receiver
import adafruit_irremote
import neopixel

''' setup the LED - used as in indicator for IR receive '''
indicator = digitalio.DigitalInOut(board.LED)
indicator.direction = digitalio.Direction.OUTPUT
indicator.value = False


''' setup the Neopixel LEDs '''
NEOPIXEL_COUNT: int = 9
leds = neopixel.NeoPixel(board.NEOPIXEL, NEOPIXEL_COUNT, brightness=0.15, auto_write=True)

# lets quickly cycle all the neopixels through a range of colors
colors: list = [0x000000, 0xFF0000, 0x00FF00, 0xFFFF00, 0x0000FF, 0xFF00FF, 0x00FFFF, 0xFFFFFF]
COLOR_COUNT: int = 8


'''
IR Data Format:
    The header has a very long pulse and a long gap.
    Each bit of the binary data starts with a marker pulse and then a long or short gap.
    Most formats include a stop bit at the end which is the same as a 'zero' bit.
'''

IR_HEADER = [9000, 4500]
IR_START = 560
IR_SHORT = 560
IR_LONG = 1670
IR_ZERO = [IR_START, IR_SHORT]
IR_ONE  = [IR_START, IR_LONG]

ir_rx = pulseio.PulseIn(board.GP1, maxlen=200, idle_state=True)
ir_tx = pulseio.PulseOut(board.GP0, frequency=38000, duty_cycle=2**15)

decoder = adafruit_irremote.GenericDecode()
encoder = adafruit_irremote.GenericTransmit(header=IR_HEADER, one=IR_ONE, zero=IR_ZERO, trail=255, debug=False)

MESSAGE_SIZE: int = 4
message = [2, 3, 0, 255] # cmd, color, duration, delay (we are only interested in the color)


def ir_received(ir) -> list:
    pulses = decoder.read_pulses(ir)
    count = len(pulses)
    if count < MESSAGE_SIZE:
        return None
    try:
        code = list(decoder.decode_bits(pulses))
        print("Received:", code)
        indicator.value = True
        time.sleep(0.5)
        indicator.value = False
        if len(code) != MESSAGE_SIZE:
            print("Bad code: ", code)
            code = None
    except Exception as e:
        print("Failed to decode:", e, "raw:", count, pulses)
        code = None
        for i in range(5):
            indicator.value = True
            time.sleep(0.05)
            indicator.value = False
            time.sleep(0.05)
    return code

def ir_transmit(ir, value):
    if type(value) is not list:
        value = [value]
    print("sending: ", value)
    encoder.transmit(ir, value)



timer = time.time() + int((5 * random.random()))

try:
    while True:
        if timer < time.time():
            timer = time.time() + int((5 * random.random()))
            ir_rx.pause()
            message[1] = random.randint(0, 7)
            ir_transmit(ir_tx, message)
            ir_rx.clear()
            ir_rx.resume()

        if len(ir_rx) > 0:
            code = ir_received(ir_rx)
            # valid code is a list of 3 values: [cmd, brightness, color]
            # brightness is a value 0..9
            # color is a value 0..7
            # cmd is not used
            if code:
                color = code[1] % COLOR_COUNT
                leds.fill(colors[color])

        time.sleep(0.05)

finally:
    leds.fill(0x000000)

